# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import unittest
import doctest

from Products.Silva.testing import (
    FunctionalLayer, suite_from_package, http, Browser)

globs = {'http': http,
         'grok': FunctionalLayer.grok,
         'Browser': Browser,
         'getRootFolder': FunctionalLayer.get_application}


def create_test(build_test_suite, name):
    test =  build_test_suite(
        name,
        globs=globs,
        optionflags=doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE)
    test.layer = FunctionalLayer
    return test


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(suite_from_package(
            'silva.core.views.tests.grok', create_test))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok
from zope.container.interfaces import IObjectRemovedEvent
import zope.component

import Acquisition
from AccessControl import getSecurityManager
from AccessControl import Unauthorized

from Products.Five.metaclass import makeClass
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

from silva.core.views.interfaces import IGrokCustomizable, ICustomizedTemplate


class TTWViewTemplate(ZopePageTemplate):
    """A template class used to generate Zope 3 views TTW"""

    meta_type="Silva TTW View Template"
    grok.implements(ICustomizedTemplate)

    manage_options = (
        ZopePageTemplate.manage_options[0],
        ) + ZopePageTemplate.manage_options[2:]

    def __init__(self, id, text=None, content_type='text/html', strict=True,
                 encoding='utf-8', view=None, permission=None, name=None):
        self.view = view
        self.permission = permission
        self.name = name
        super(TTWViewTemplate, self).__init__(
            id, text, content_type, encoding, strict)

    def __call__(self, *args):
        #XXX raise a sensible exception if context and request are
        # omitted, IOW, if someone tries to render the template not as
        # a view.
        sm = getSecurityManager()
        if self.permission:
            if not sm.checkPermission(self.permission, args[0]):
                raise Unauthorized('The current user does not have the '
                                   'required "%s" permission'
                                   % self.permission)

        class_ = makeClass(
            'FrankensteinTTWTemplate',
            (TTWViewTemplateRenderer, self.view),
            {'__name__': self.name,
             '__view_name__': self.name,
             'module_info': FakeModuleInfoForGrok(self.view.__module__)})

        return class_(self, self.view, args)

    # overwrite Shared.DC.Scripts.Binding.Binding's before traversal
    # hook that would prevent to look up views for instances of this
    # class.
    def __before_publishing_traverse__(self, self2, request):
        pass


class FakeModuleInfoForGrok(object):

    def __init__(self, path):
        self.package_dotted_name = path


class TTWViewTemplateRenderer(Acquisition.Implicit):
    """The view object for the TTW View Template.

    When a TTWViewTemplate-based view is looked up, an object of this
    class is instantiated.  It holds a reference to the
    TTWViewTemplate object which it will use in the render process
    (__call__).
    """
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, template, view, args):
        self.template = template
        view.__init__(self, *args)

    def __call__(self, *args, **kwargs):
        """Render the TTWViewTemplate-based view.
        """
        return self.render(*args, **kwargs)

    def render(self, *args, **kwargs):
        """Render the view
        """
        # we need to override the template's context and request as
        # they generally point to the wrong objects (a template's
        # context usually is what it was acquired from, which isn't
        # what the context is for a view template).
        bound_names = {'context': self.context,
                       'request': self.request,
                       'view': self}

        if IGrokCustomizable.providedBy(self):
            bound_names.update(self.default_namespace())
            self.update()
            if self.request.response.getStatus() in (302, 303):
                return

        return self.template._exec(bound_names, args, kwargs)



@grok.subscribe(TTWViewTemplate, IObjectRemovedEvent)
def unregisterViewWhenZPTIsDeleted(template, event):
    components = zope.component.getSiteManager(template)
    for view in components.registeredAdapters():
        if view.factory == template:
            components.unregisterAdapter(
                view.factory, view.required, view.provided, view.name)
            break



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from silva.core import conf as silvaconf

silvaconf.extensionName('silva.core.views')
silvaconf.extensionTitle('Silva Core Views')
silvaconf.extensionSystem()

import zope.deferredimport
zope.deferredimport.deprecated(
    'Forms moved into silva.core.forms.',
    forms = 'silva.core.forms.forms',
    z3cforms = 'silva.core.forms.z3cforms')



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from grokcore.view.interfaces import IGrokView
from grokcore.viewlet.interfaces import IViewletManager as IBaseViewletManager
from zope.contentprovider.interfaces import (
    IContentProvider as IBaseContentProvider)
from zope.interface import Interface, Attribute
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.traversing.browser.interfaces import IAbsoluteURL
from zope.viewlet.interfaces import IViewlet as IBaseViewlet

import zope.deferredimport
zope.deferredimport.deprecated(
    'Please import from silva.core.smi instead,'
    'this import will be removed in Silva 2.4',
    ISMIView='silva.core.smi.interfaces:ISMIView',
    ISMITab='silva.core.smi.interfaces:ISMITab',)


class ITestRequest(IBrowserRequest):
    """Marker interface to mark a TestRequest.
    """


# Preview layer

class INonCachedLayer(IBrowserRequest):
    """Layer that prevent its content to be cached by the HTTP protocol.
    """


class IPreviewLayer(INonCachedLayer):
    """This layer enable the fact to display preview version instead
    of public version.
    """

# View

class IGrokCustomizable(Interface):
    """A grok template which can be customized with a TTW template.

    Conviently it's a sub-set of a GrokView: it's need.
    """

    def update():
        """Update method which have to be called before rendering the
        template.
        """

    def default_namespace():
        """Return default namespace values.
        """


class ITemplateNotCustomizable(Interface):
    """Marker interface to put on view/template that you don't people be able
    to customize.
    """


class IView(IGrokView):
    """A view in Silva.
    """

    is_preview = Attribute(u"Boolean which say if you're in preview mode.")
    content = Attribute(u"Version of the content to render.")


class IZMIView(IGrokView):
    """A view in ZMI.
    """

class IContentProvider(IBaseContentProvider, IGrokCustomizable):
    """A customizable Content Provider.
    """


class IViewletManager(IBaseViewletManager, IGrokCustomizable):
    """A customizable Viewlet Manager.
    """


class IViewlet(IBaseViewlet, IGrokCustomizable):
    """A customizable Viewlet.
    """

class ITemplateCustomizable(IGrokCustomizable):
    """A page that you can customize.
    """

# TTW Templates

class ICustomizedTemplate(ITemplateCustomizable):
    """A through the web template.
    """

# URL management / with preview

class ISilvaURL(IAbsoluteURL):
    """Extends the absolute URL mechanism to support preview and
    public URLs.
    """

    def preview():
        """Return URL for preview.
        """

# Adapters

class IVirtualSite(Interface):
    """Adapter on a Zope request to retrieve the root object of the
    current site. If a Virtual Host Monster is used, the root object
    for your site might not be your Silva root, but a publication or a
    folder.
    """

    def get_root():
        """Return the root object of the current site, which can be
        either the virtual root object or the Silva root depending if
        Virtual Host Monster rewriting is used for the request or not.
        """

    def get_root_url():
        """Return the URL of the root object of the current site.
        """

    def get_silva_root():
        """Return the Silva root.
        """

    def get_virtual_root():
        """Return the virtual root object defined by the Virtual Host
        Monster or None if the Virtual Host Monster was not used.
        """

    def get_virtual_path():
        """Return the path (as a tuple of identifier) to the virtual
        defined by the Virtual Host Monster or None if the Virtual
        Host Monster was not used.
        """


class IHTTPResponseHeaders(Interface):
    """Adapter on a context and a request which is used to set HTTP
    headers on the response.

    Headers can be cache control settings, ...
    """

    def cache_headers():
        """ Set the cache and Last modified settings.
        """

    def other_headers(headers):
        """ Set other headers.
        """

    def __call__(**headers):
        """Set headers on the response

        method should accept **kwargs argument to override headers
        """





# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok
from zope.site.hooks import getSite
from zope.publisher.interfaces.http import IHTTPRequest
from zope.traversing.browser import absoluteURL

from silva.core.views.interfaces import IVirtualSite


class VirtualSite(grok.Adapter):
    grok.implements(IVirtualSite)
    grok.provides(IVirtualSite)
    grok.context(IHTTPRequest)

    def __init__(self, request):
        self.request = request

    def get_root(self):
        root = self.get_virtual_root()
        if root is None:
            return self.get_silva_root()
        return root

    def get_root_url(self):
        root = self.get_root()
        if root is not None:
            return absoluteURL(root, self.request)
        return u''

    def get_silva_root(self):
        # Return Silva root, using getSite and acquisition
        site = getSite()
        if site:
            return site.get_root()
        return None

    def get_virtual_root(self):
        root_path = self.get_virtual_path()
        if root_path is None:
            return None

        return getSite().restrictedTraverse(root_path, None)

    def get_virtual_path(self):
        try:
            root_path = self.request['VirtualRootPhysicalPath']
        except (AttributeError, KeyError):
            root_path =  None

        return root_path




# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from Products.Silva.testing import FunctionalLayer, http
from silva.core.interfaces.auth import IAccessSecurity

import unittest

AUTH = {
    'Authorization': 'Basic manager:manager'}

PUBLIC_HEADERS_EXPECTED = {
    'Content-Length': '0',
    'Content-Type': 'text/html;charset=utf-8',
    'Cache-Control': 'max-age=86400, must-revalidate'}
PRIVATE_HEADERS_EXPECTED = {
    'Content-Length': '0',
    'Expires': 'Mon, 26 Jul 1997 05:00:00 GMT',
    'Content-Type': 'text/html;charset=utf-8',
    'Pragma': 'no-cache',
    'Cache-Control': 'no-cache, must-revalidate, post-check=0, pre-check=0'}


class HEADTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()

    def check_headers(self, method, path, headers, expected_headers):
        # XXX use infrae.testbrowser
        request = '%s %s HTTP/1.1' % (method, path)
        for header in headers.items():
            request += '\r\n%s: %s' % header
        self.response = http(request, parsed=True)
        self.failUnless(self.response.getStatus(), 200)
        response_headers = self.response.getHeaders()
        for key, value in expected_headers.items():
            reply_value = response_headers.get(key, None)
            self.assertEquals(
                reply_value, value,
                'Invalid header for "%s", expected: "%s", was: "%s"' %
                (key, value, reply_value))

    def set_private(self, context):
        IAccessSecurity(context).minimum_role = 'Authenticated'

    def assertEmptyResponse(self):
        self.assertEquals(
            "", self.response.getBody(), "response should be empty")


class SilvaHEADTestCase(HEADTestCase):

    def test_root(self):
        self.check_headers('HEAD', '/root', {}, PUBLIC_HEADERS_EXPECTED)
        self.assertEmptyResponse()

    def test_root_auth(self):
        self.check_headers('HEAD', '/root', AUTH, PUBLIC_HEADERS_EXPECTED)
        self.assertEmptyResponse()

    def test_root_auth_private(self):
        self.set_private(self.root)
        self.check_headers('HEAD', '/root', AUTH, PRIVATE_HEADERS_EXPECTED)
        self.assertEmptyResponse()

    def test_publication(self):
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addPublication('publication', 'Publication')
        self.check_headers(
            'HEAD', '/root/publication', {}, PUBLIC_HEADERS_EXPECTED)
        self.assertEmptyResponse()

    def test_publication_auth(self):
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addPublication('publication', 'Publication')
        self.check_headers(
            'HEAD', '/root/publication', AUTH, PUBLIC_HEADERS_EXPECTED)
        self.assertEmptyResponse()

    def test_publication_auth_private(self):
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addPublication('publication', 'Publication')
        self.set_private(self.root.publication)
        self.check_headers(
            'HEAD', '/root/publication', AUTH, PRIVATE_HEADERS_EXPECTED)
        self.assertEmptyResponse()


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SilvaHEADTestCase))
    return suite



# Copyright (c) 2008-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$
"""

 We grok this test file:
    >>> grok('silva.core.views.tests.grok.view_cache_headers')

 Now we add a folder:

    >>> root = getRootFolder()
    >>> factory = root.manage_addProduct['Silva']
    >>> _ = factory.manage_addFolder('myfolder', 'My Folder')
    >>> root.myfolder
    <Folder at /root/myfolder>

 When requesting the view the response should include cache control
 headers:

    >>> browser = Browser()
    >>> browser.open('http://localhost/root/myfolder/mytestview')
    >>> browser.status
    '200 OK'
    >>> browser.contents
    'This is a view!'
    >>> browser.headers.has_key('cache-control')
    True
    >>> browser.headers['Cache-Control']
    'max-age=86400, must-revalidate'

 We can do HEAD requests:

    >>> reply = http('HEAD /root/myfolder/mytestview HTTP/1.1', parsed=True)
    >>> reply.getHeaders()
    {'Content-Length': '0',
     'Content-Type': 'text/html;charset=utf-8',
     'Cache-Control': 'max-age=86400, must-revalidate'}
    >>> reply.getBody()
    ''

 We now create a protected folder:

    >>> from silva.core.interfaces.auth import IAccessSecurity
    >>> IAccessSecurity(root.myfolder).minimum_role = 'Authenticated'

 If the view is private you should not have cache headers:

    >>> browser.addHeader("Authorization", 'Basic manager:manager')
    >>> browser.open('http://localhost/root/myfolder/mytestview')
    >>> browser.status
    '200 OK'
    >>> browser.contents
    'This is a view!'
    >>> browser.headers.has_key('cache-control')
    True
    >>> browser.headers['Cache-Control']
    'no-cache, must-revalidate, post-check=0, pre-check=0'
    >>> browser.headers.has_key('pragma')
    True
    >>> browser.headers['Pragma']
    'no-cache'

"""

from Products.Silva.Folder import Folder
from silva.core.views import views as silvaviews
from five import grok


class MyFolderView(silvaviews.View):
    grok.name('mytestview')
    grok.context(Folder)

    def render(self):
        return "This is a view!"



# Copyright (c) 2002-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

# Zope 3
from zope.component import getMultiAdapter
from zope.interface import implements

# Zope 2
from OFS.interfaces import ITraversable
from Products.Five import BrowserView
from Acquisition import aq_parent

from silva.core.interfaces import IRoot, IContent
from silva.core.views.interfaces import IPreviewLayer, ISilvaURL


def minimize(string):
    """Minize a string to 50 characters.
    """
    string = string.strip()
    if len(string) > 50:
        return string[:47].strip() + '...'
    return string


class AbsoluteURL(BrowserView):
    """An adapter for Zope3-style absolute_url using Zope2 methods

    (original: zope.traversing.browser.absoluteurl)
    """
    implements(ISilvaURL)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self._preview_ns = '++preview++'

        def title():
            if IPreviewLayer.providedBy(self.request):
                return context.get_previewable().get_short_title()
            return context.get_short_title()

        self.title = title

    def url(self, preview=False):
        path = list(self.context.getPhysicalPath())
        # Insert back the preview namespace. Maybe there is a better
        # way to do it, but have to do it by hand here since
        # ZPublisher.Request doesn't implements its interfaces
        # properly.
        if preview is True:
            root = self.context.get_root()
            root_path = root.getPhysicalPath()
            virtual_path = self.request.other.get(
                'VirtualRootPhysicalPath', ('',))
            preview_pos = max(len(root_path), len(virtual_path))
            path.insert(preview_pos, self._preview_ns)
        return self.request.physicalPathToURL(path)

    def preview(self):
        return self.url(preview=True)

    def __str__(self):
        return self.url(preview=IPreviewLayer.providedBy(self.request))

    __call__ = __repr__ = __unicode__ = __str__

    def breadcrumbs(self):
        container = aq_parent(self.context)
        name = minimize(self.title())

        def isVirtualHostRoot():
            path = self.context.getPhysicalPath()
            virtualPath = self.request.physicalPathToVirtualPath(path)
            return not virtualPath

        if (container is None or
            IRoot.providedBy(self.context) or
            isVirtualHostRoot() or
            not ITraversable.providedBy(container)):
            return ({'name': name, 'url': self.__str__()},)

        base = tuple(getMultiAdapter(
                (container, self.request), name='absolute_url').breadcrumbs())

        if not (IContent.providedBy(self.context) and self.context.is_default()):
            base += ({'name': name, 'url': self.__str__()},)

        return base


class VersionAbsoluteURL(AbsoluteURL):
    """AbsoluteURL for a version
    """

    def __init__(self, context, request):
        # Set versioned content as context
        self.context = context.get_content()
        self.request = request
        self._preview_ns = '++preview++' + context.getId()
        self.title = lambda: context.get_short_title()


class ErrorAbsoluteURL(AbsoluteURL):
    """AbsoluteURL for a version
    """

    def __init__(self, context, request):
        # Set first Silva object as context
        self.context = context.get_silva_object()
        self.request = request
        self._preview_ns = '++preview++'


class TestAbsoluteURL(BrowserView):
    """An absolute URL provider for TestRequest. This is mainly to get
    test working.
    """

    implements(ISilvaURL)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def url(self, preview=False):
        is_root = True
        if self.context is not None:
            path = list(self.context.getPhysicalPath())
            is_root = len(path) < (preview is True and 4 or 3)
            if preview is True:
                path.insert(2, '++preview++')
        if is_root:
            return u'http://localhost/root'
        return u'http://localhost' + '/'.join(path)

    def preview(self):
        return self.url(preview=True)

    def __str__(self):
        return self.url()

    __call__ = __repr__ = __unicode__ = __str__

    def breadcrumbs(self):
        return tuple()



# Copyright (c) 2002-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok
from zope.publisher.interfaces.browser import IBrowserRequest
from silva.core.interfaces import ISilvaObject
from silva.core.interfaces.auth import IAccessSecurity
from silva.core.views.interfaces import IHTTPResponseHeaders, INonCachedLayer


class ResponseHeaderHandler(grok.MultiAdapter):
    """Default class to implement HTTP header settings.
    """
    grok.baseclass()

    def __init__(self, request, context):
        self.context = context
        self.request = request
        self.response = request.response

    def cache_headers(self):
        pass

    def other_headers(self, headers):
        if 'Content-Type' not in headers:
            headers['Content-Type'] = 'text/html;charset=utf-8'
        for key, value in headers.items():
            self.response.setHeader(key, value)

    def set_headers(self, **headers):
        """Set the headers on the response. Called directly to
        implement HEAD requests as well.
        """
        self.cache_headers()
        self.other_headers(headers)
        return ''

    def __call__(self, **headers):
        return self.set_headers(**headers)

    def disable_cache(self):
        self.response.setHeader(
            'Cache-Control',
            'no-cache, must-revalidate, post-check=0, pre-check=0')
        self.response.setHeader('Expires', 'Mon, 26 Jul 1997 05:00:00 GMT')
        self.response.setHeader('Pragma', 'no-cache')


class ErrorHeaders(ResponseHeaderHandler):
    """Errors are not cached.
    """
    grok.adapts(IBrowserRequest, Exception)
    grok.implements(IHTTPResponseHeaders)
    grok.provides(IHTTPResponseHeaders)

    def cache_headers(self):
        self.disable_cache()


class HTTPResponseHeaders(ResponseHeaderHandler):
    """Regular Silva content headers.
    """
    grok.adapts(IBrowserRequest, ISilvaObject)
    grok.implements(IHTTPResponseHeaders)
    grok.provides(IHTTPResponseHeaders)

    max_age = 86400

    def cache_headers(self):
        if self.__is_private() or self.__is_preview():
            # No cache when private
            self.disable_cache()
        else:
            self.response.setHeader(
                'Cache-Control',
                'max-age=%d, must-revalidate' % self.max_age)

    def __is_preview(self):
        return INonCachedLayer.providedBy(self.request)

    def __is_private(self):
        return IAccessSecurity(self.context).minimum_role is not None



# Copyright (c) 2002-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok
from zope.component import getMultiAdapter
from zope.interface import alsoProvides
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.traversing.interfaces import ITraversable

from Acquisition.interfaces import IAcquirer
from ZPublisher.BaseRequest import DefaultPublishTraverse

from silva.core.interfaces import IPublication
from silva.core.views.interfaces import IHTTPResponseHeaders
from silva.core.views.interfaces import IPreviewLayer


class UseParentByAcquisition(object):
    """Request traverseName which query the ITraversable adapter always
    do a __of__ of the result of the queried adapter to get the new
    context object to use afterward. It is annoying if you don't want
    to change of context object, as if you return the same object you
    got in input it will have __of__ of himself.

    Returning an instance of this object will prevent that behavior
    and permit you to stay on the same context object.
    """
    grok.implements(IAcquirer)

    def __of__(self, obj):
        return obj


class PreviewTraversable(grok.MultiAdapter):
    """Traverser to display versioned contents in SMI preview.

    Add the preview layer on the request if needed.
    """
    grok.adapts(IPublication, IBrowserRequest)
    grok.implements(ITraversable)
    grok.name('preview')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def traverse(self, name, remaining):
        if not IPreviewLayer.providedBy(self.request):
            alsoProvides(self.request, IPreviewLayer)
        if name:
            self.request.other['SILVA_PREVIEW_NAME'] = name
        return UseParentByAcquisition()



class SilvaPublishTraverse(DefaultPublishTraverse):

    def browserDefault(self, request):
        # We don't want to lookup five views if we have other than a
        # GET or POST request.
        if request.method in ('GET', 'HEAD',):
            response_headers = getMultiAdapter(
                (self.request, self.context), IHTTPResponseHeaders)
            response_headers()
        if request.method in ('GET', 'POST',):
            return super(SilvaPublishTraverse, self).browserDefault(request)
        if request.method == 'HEAD':
            return SilvaHEADRequest(self.context, request), ('HEAD',)
        return self.context, tuple()


class SilvaHEADRequest(object):
    """Reply to an HEAD request on a Silva object.
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def HEAD(self, *args, **kw):
        """Return empty response body
        """
        return u''




# Copyright (c) 2002-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import unittest

from zope.interface.verify import verifyObject
from zope.interface import alsoProvides
from zope.traversing.browser.interfaces import IAbsoluteURL
from zope import component

from Products.Silva.testing import FunctionalLayer

from silva.core.views.interfaces import ISilvaURL
from silva.core.views.interfaces import IPreviewLayer


def enable_preview(content):
    """Enable preview mode.
    """
    if not IPreviewLayer.providedBy(content.REQUEST):
        alsoProvides(content.REQUEST, IPreviewLayer)


class PublicationAbsoluteURLTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addPublication('publication', u'Test Publication')

    def test_url(self):
        url = component.getMultiAdapter((self.root.publication, self.root.REQUEST), ISilvaURL)
        self.assertTrue(verifyObject(ISilvaURL, url))
        self.assertTrue(ISilvaURL.extends(IAbsoluteURL))

        self.assertEqual(
            str(url),
            'http://localhost/root/publication')
        self.assertEqual(
            url(),
            'http://localhost/root/publication')
        self.assertEqual(
            url.preview(),
            'http://localhost/root/++preview++/publication')

        enable_preview(self.root)
        self.assertEqual(
            str(url),
            'http://localhost/root/++preview++/publication')
        self.assertEqual(
            url(),
            'http://localhost/root/++preview++/publication')

    def test_breadcrumbs(self):
        url = component.getMultiAdapter((self.root.publication, self.root.REQUEST), ISilvaURL)

        self.assertEqual(
            url.breadcrumbs(),
            ({'url': 'http://localhost/root', 'name': 'root'},
             {'url': 'http://localhost/root/publication', 'name': u'Test Publication'}))

        enable_preview(self.root)
        self.assertEqual(
            url.breadcrumbs(),
            ({'url': 'http://localhost/root/++preview++', 'name': 'root'},
             {'url': 'http://localhost/root/++preview++/publication', 'name': u'Test Publication'}))

    def test_traverse(self):
        url = self.root.publication.restrictedTraverse('@@absolute_url')
        self.assertTrue(verifyObject(ISilvaURL, url))


class RootAbsoluteURLTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()

    def test_url(self):
        url = component.getMultiAdapter((self.root, self.root.REQUEST), ISilvaURL)
        self.assertTrue(verifyObject(ISilvaURL, url))
        self.assertTrue(ISilvaURL.extends(IAbsoluteURL))

        self.assertEqual(
            str(url),
            'http://localhost/root')
        self.assertEqual(
            url(),
            'http://localhost/root')
        self.assertEqual(
            url.preview(),
            'http://localhost/root/++preview++')

        enable_preview(self.root)
        self.assertEqual(
            str(url),
            'http://localhost/root/++preview++')
        self.assertEqual(
            url(),
            'http://localhost/root/++preview++')

    def test_breadcrumbs(self):
        url = component.getMultiAdapter((self.root, self.root.REQUEST), ISilvaURL)

        self.assertEqual(
            url.breadcrumbs(),
            ({'url': 'http://localhost/root', 'name': 'root'},))

        enable_preview(self.root)
        self.assertEqual(
            url.breadcrumbs(),
            ({'url': 'http://localhost/root/++preview++', 'name': 'root'},))

    def test_traverse(self):
        url = self.root.restrictedTraverse('@@absolute_url')
        self.assertTrue(verifyObject(ISilvaURL, url))


class VersionedContentAbsoluteURLTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addPublication('publication', u'Test Publication')
        factory = self.root.publication.manage_addProduct['Silva']
        factory.manage_addFolder('folder', u'Folder')
        factory = self.root.publication.folder.manage_addProduct['Silva']
        factory.manage_addLink('link', u'Link')

    def test_url(self):
        content = self.root.publication.folder.link
        url = component.getMultiAdapter((content, self.root.REQUEST), ISilvaURL)
        self.assertTrue(verifyObject(ISilvaURL, url))
        self.assertTrue(ISilvaURL.extends(IAbsoluteURL))

        self.assertEqual(
            str(url),
            'http://localhost/root/publication/folder/link')
        self.assertEqual(
            url(),
            'http://localhost/root/publication/folder/link')
        self.assertEqual(
            url.preview(),
            'http://localhost/root/++preview++/publication/folder/link')

        enable_preview(self.root)
        self.assertEqual(
            str(url),
            'http://localhost/root/++preview++/publication/folder/link')
        self.assertEqual(
            url(),
            'http://localhost/root/++preview++/publication/folder/link')

    def test_breadcrumbs(self):
        content = self.root.publication.folder.link
        url = component.getMultiAdapter((content, self.root.REQUEST), ISilvaURL)

        self.assertEqual(
            url.breadcrumbs(),
            ({'url': 'http://localhost/root', 'name': u'root'},
             {'url': 'http://localhost/root/publication', 'name': u'Test Publication'},
             {'url': 'http://localhost/root/publication/folder', 'name': u'Folder'},
             {'url': 'http://localhost/root/publication/folder/link', 'name': u'link'}))

        enable_preview(self.root)
        self.assertEqual(
            url.breadcrumbs(),
            ({'url': 'http://localhost/root/++preview++', 'name': u'root'},
             {'url': 'http://localhost/root/++preview++/publication', 'name': u'Test Publication'},
             {'url': 'http://localhost/root/++preview++/publication/folder', 'name': u'Folder'},
             {'url': 'http://localhost/root/++preview++/publication/folder/link', 'name': u'Link'}))

    def test_traverse(self):
        content = self.root.publication.folder.link
        url = content.restrictedTraverse('@@absolute_url')
        self.assertTrue(verifyObject(ISilvaURL, url))


class VersionContentAbsoluteURLTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addPublication('publication', u'Test Publication')
        factory = self.root.publication.manage_addProduct['Silva']
        factory.manage_addFolder('folder', u'Folder')
        factory = self.root.publication.folder.manage_addProduct['Silva']
        factory.manage_addLink('link', u'Link with like a completely super long title to test them')

    def test_url(self):
        content = self.root.publication.folder.link.get_editable()
        url = component.getMultiAdapter((content, self.root.REQUEST), ISilvaURL)
        self.assertTrue(verifyObject(ISilvaURL, url))
        self.assertTrue(ISilvaURL.extends(IAbsoluteURL))

        self.assertEqual(
            str(url),
            'http://localhost/root/publication/folder/link')
        self.assertEqual(
            url(),
            'http://localhost/root/publication/folder/link')
        self.assertEqual(
            url.preview(),
            'http://localhost/root/++preview++0/publication/folder/link')

        enable_preview(self.root)
        self.assertEqual(
            str(url),
            'http://localhost/root/++preview++0/publication/folder/link')
        self.assertEqual(
            url(),
            'http://localhost/root/++preview++0/publication/folder/link')

    def test_breadcrumbs(self):
        content = self.root.publication.folder.link.get_editable()
        url = component.getMultiAdapter((content, self.root.REQUEST), ISilvaURL)

        self.assertEqual(
            url.breadcrumbs(),
            ({'url': 'http://localhost/root', 'name': u'root'},
             {'url': 'http://localhost/root/publication', 'name': u'Test Publication'},
             {'url': 'http://localhost/root/publication/folder', 'name': u'Folder'},
             {'url': 'http://localhost/root/publication/folder/link',
              'name': u'Link with like a completely super long title to...'}))

        enable_preview(self.root)
        self.assertEqual(
            url.breadcrumbs(),
            ({'url': 'http://localhost/root/++preview++', 'name': 'root'},
             {'url': 'http://localhost/root/++preview++/publication', 'name': u'Test Publication'},
             {'url': 'http://localhost/root/++preview++/publication/folder', 'name': u'Folder'},
             {'url': 'http://localhost/root/++preview++0/publication/folder/link',
              'name': u'Link with like a completely super long title to...'}))

    def test_traverse(self):
        content = self.root.publication.folder.link.get_editable()
        url = content.restrictedTraverse('@@absolute_url')
        self.assertTrue(verifyObject(ISilvaURL, url))


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(PublicationAbsoluteURLTestCase))
    suite.addTest(unittest.makeSuite(RootAbsoluteURLTestCase))
    suite.addTest(unittest.makeSuite(VersionedContentAbsoluteURLTestCase))
    suite.addTest(unittest.makeSuite(VersionContentAbsoluteURLTestCase))
    return suite





# -*- coding: utf-8 -*-
# Copyright (c) 2002-2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from Acquisition import aq_base

from five import grok
from infrae.layout import Layout as BaseLayout
from infrae.layout import Page as BasePage
from infrae.layout.interfaces import IPage
from zope import component
from zope.cachedescriptors.property import CachedProperty
from zope.viewlet.interfaces import IViewletManager
import zope.deferredimport

from silva.core.interfaces import ISilvaObject
from silva.core.views.interfaces import IContentProvider, IViewlet
from silva.core.views.interfaces import IZMIView
from silva.core.views.interfaces import IPreviewLayer
from silva.core.views.interfaces import IView, IHTTPResponseHeaders

zope.deferredimport.deprecated(
    'SMIView moved to silva.core.smi. '
    'Consider using a SMIPage instead. It will be removed in Silva 2.4',
    SMIView='silva.core.smi.smi:SMIView')
zope.deferredimport.deprecated(
    'SMIPortletManager moved to silva.core.smi.smi. '
    'Please update your import. This one will be removed in Silva 2.4',
    SMIPortletManager='silva.core.smi.smi:SMIPortletManager')


# Simple views


class HTTPHeaderView(object):
    """Support to set HTTP headers and support HEAD requests on views
    objects.

    This is not aimed to be use directly.
    """

    def getPhysicalPath(self):
        return self.context.getPhysicalPath() + (self.__name__,)

    def publishTraverse(self, request, name):
        if request.method == name and hasattr(self, name):
            return getattr(self, name)
        return super(HTTPHeaderView, self).publishTraverse(request, name)

    def browserDefault(self, request):
        if request.method in ('HEAD',):
            if hasattr(self, request.method):
                return self, (request.method,)
        return super(HTTPHeaderView, self).browserDefault(request)

    def setHTTPHeaders(self):
        headers = component.queryMultiAdapter(
            (self.request, self.context), IHTTPResponseHeaders)
        if headers is not None:
            headers.set_headers()

    def HEAD(self):
        """Reply to HEAD requests.
        """
        self.setHTTPHeaders()
        return ''

    def __call__(self):
        """Render the view.
        """
        self.setHTTPHeaders()
        return super(HTTPHeaderView, self).__call__()


class ZMIView(HTTPHeaderView, grok.View):
    """View in ZMI.
    """
    grok.baseclass()
    grok.require('zope2.ViewManagementScreens')
    grok.implements(IZMIView)


class Layout(BaseLayout):
    """A layout object.
    """
    grok.baseclass()
    grok.context(ISilvaObject)


class Page(HTTPHeaderView, BasePage):
    """A page class using a layout to render itself.
    """
    grok.baseclass()
    grok.context(ISilvaObject)
    grok.require('zope2.View')


class View(HTTPHeaderView, grok.View):
    """View on Silva object, support view and preview
    """
    grok.baseclass()
    grok.context(ISilvaObject)
    grok.implements(IView)
    grok.name(u'content.html')
    grok.require('zope2.View')

    @CachedProperty
    def is_preview(self):
        return IPreviewLayer.providedBy(self.request)

    @CachedProperty
    def content(self):
        preview_name = self.request.other.get('SILVA_PREVIEW_NAME', None)
        if (preview_name is not None and
            hasattr(aq_base(self.context), preview_name)):
            return getattr(self.context, preview_name)
        version = None
        if self.is_preview:
            version = self.context.get_previewable()
        if version is None:
            version = self.context.get_viewable()
        return version

    def namespace(self):
        return {'content': self.content}


class ViewletLayoutSupport(object):
    """This add layout on the object and namespace if the view is an
    page.
    """

    def __init__(self, *args):
        super(ViewletLayoutSupport, self).__init__(*args)
        self.layout = None
        if IPage.providedBy(self.view):
            self.layout = self.view.layout

    def default_namespace(self):
        namespace = super(ViewletLayoutSupport, self).default_namespace()
        if self.layout:
            namespace['layout'] = self.layout
        return namespace


class ViewletManager(ViewletLayoutSupport, grok.ViewletManager):
    """A viewlet manager in Silva.
    """
    grok.baseclass()
    grok.context(ISilvaObject)
    grok.implements(IViewletManager)


class ContentProvider(ViewletManager):
    """A content provider in Silva. In fact it's just a viewlet
    manager...
    """
    grok.baseclass()
    grok.context(ISilvaObject)
    grok.implements(IContentProvider)

    def default_namespace(self):
        namespace = super(ContentProvider, self).default_namespace()
        namespace['provider'] = self
        return namespace


class Viewlet(ViewletLayoutSupport, grok.Viewlet):
    """A viewlet in Silva
    """
    grok.baseclass()
    grok.context(ISilvaObject)
    grok.implements(IViewlet)
    grok.require('zope2.View')


__all__ = [
    'View', 'ZMIView', 'Layout', 'Page',
    'ContentProvider', 'ViewletManager', 'Viewlet']
